@@ -1496,6 +1496,7 @@ int Dbtup::interpreterNextLab(Signal* signal,
 	    // word read. Thus we set the register to be a 32 bit register.
 	    /* ------------------------------------------------------------- */
 	    TregMemBuffer[theRegister] = 0x50;
+            // arithmetic conversion if big-endian
             * (Int64*)(TregMemBuffer+theRegister+2) = TregMemBuffer[theRegister+1];
 	  } else if (TnoDataRW == 3) {
 	    /* ------------------------------------------------------------- */
@@ -1554,6 +1555,11 @@ int Dbtup::interpreterNextLab(Signal* signal,
 	  Tlen = TattrNoOfWords + 1;
 	  if (Toptype == ZUPDATE) {
 	    if (TattrNoOfWords <= 2) {
+              if (TattrNoOfWords == 1) {
+                // arithmetic conversion if big-endian
+                TdataForUpdate[1] = *(Int64*)&TregMemBuffer[theRegister + 2];
+                TdataForUpdate[2] = 0;
+              }
 	      if (TregType == 0) {
 		/* --------------------------------------------------------- */
 		// Write a NULL value into the attribute
