@@ -19,7 +19,7 @@ static int nope = 0;		/* for use in asserts; shuts lint up */
 
 /* macros for manipulating states, small version */
 #define	states	long
-#define	states1	states		/* for later use in regexec() decision */
+#define	states1	long		/* for later use in regexec() decision. Ensure Win64 definition is correct.*/
 #define	CLEAR(v)	((v) = 0)
 #define	SET0(v, n)	((v) &= ~((states) 1 << (n)))
 #define	SET1(v, n)	((v) |= (states) 1 << (n))
