@@ -61,13 +61,6 @@ public:
   */
   LEX_STRING m_sroutines_key;
 
-  sp_name(LEX_STRING name)
-    : m_name(name)
-  {
-    m_db.str= m_qname.str= m_sroutines_key.str= 0;
-    m_db.length= m_qname.length= m_sroutines_key.length= 0;
-  }
-
   sp_name(LEX_STRING db, LEX_STRING name)
     : m_db(db), m_name(name)
   {
@@ -101,8 +94,6 @@ public:
   {}
 };
 
-sp_name *
-sp_name_current_db_new(THD *thd, LEX_STRING name);
 
 bool
 check_routine_name(LEX_STRING name);
@@ -355,7 +346,6 @@ private:
 
   MEM_ROOT *m_thd_root;		// Temp. store for thd's mem_root
   THD *m_thd;			// Set if we have reset mem_root
-  char *m_thd_db;		// Original thd->db pointer
 
   sp_pcontext *m_pcont;		// Parse context
   List<LEX> m_lex;		// Temp. store for the other lex
@@ -520,7 +510,10 @@ public:
   virtual ~sp_lex_keeper()
   {
     if (m_lex_resp)
+    {
+      lex_end(m_lex);
       delete m_lex;
+    }
   }
 
   /*
@@ -1166,6 +1159,6 @@ Item *
 sp_prepare_func_item(THD* thd, Item **it_addr);
 
 bool
-sp_eval_expr(THD *thd, Field *result_field, Item *expr_item);
+sp_eval_expr(THD *thd, Field *result_field, Item **expr_item_ptr);
 
 #endif /* _SP_HEAD_H_ */
